import { Resources } from "@gitbeaker/core";
import { Gitlab } from "@gitbeaker/node";
import { ReleaseSchema } from "@gitbeaker/core/dist/types/resources/Releases";
import { MagmaRelease } from "../types/release";
import remember from "./redis";

export let gitlab: Resources.Gitlab;

export function setupGitlab() {
    gitlab = new Gitlab({
        token: process.env.GITLAB_TOKEN,
        host: process.env.GITLAB_HOST,
    });
}

export async function getArchived(repo: string) {
    return await remember(
        `archived:${repo}`,
        async () => {
            const {archived} = await gitlab.Projects.show( encodeURI(`magmafoundation/${repo}`));
            return archived;
        },
        60 * 60 * 24,
    );
}

export async function getReleases(repo: string): Promise<Array<ReleaseSchema>> {
    return await remember<Array<ReleaseSchema>>(
        `gh:releases:${repo}`,
        async () => {
            console.log(`Fetching releases for ${repo} ...`);
            try {
                return await gitlab.Releases.all(
                    encodeURI(`magmafoundation/${repo}`),
                );
            } catch (error) {
                console.log(error);
                return null;
            }
        },
    );
}

export async function getReleaseByTag(
    repo: string,
    tag: string,
): Promise<ReleaseSchema> {
    return await remember<ReleaseSchema>(
        `gh:release:${repo}:${tag}`,
        async () => {
            console.log(`Fetching release for ${repo} ${tag} ...`);
            try {
                return await gitlab.Releases.show(
                    encodeURI(`magmafoundation/${repo}`),
                    tag,
                );
            } catch (error) {
                console.log("Failed to fetch release for", repo, tag);
                return null;
            }
        },
    );
}

export async function getMagmaReleases(
    repo: string,
): Promise<Array<MagmaRelease>> {
    const magmaReleases: Array<MagmaRelease> = [];

    const releases = await getReleases(repo);
    if(!releases) return magmaReleases;

    let archived = await getArchived(repo);
    if(!archived) archived = false;

    for (const release of releases) {
        const gitCommitUrl = `${process.env.GITLAB_HOST}/magmafoundation/${repo}/-/commit/${release.tag_name}`;

        if (release.assets.links[0].url.includes("server")) {
            magmaReleases.push({
                name: release.tag_name,
                tag_name: release.tag_name,
                created_at: release.created_at,
                link: release.assets.links[0].url,
                installer_link: release.assets.links[1]?.url,
                git_commit_url: gitCommitUrl,
                archived: archived,
            });
        } else {
            magmaReleases.push({
                name: release.tag_name,
                tag_name: release.tag_name,
                created_at: release.created_at,
                link: release.assets.links[0].url,
                installer_link: release.assets.links[0].url,
                git_commit_url: gitCommitUrl,
                archived: archived,
            });
        }
    }

    return magmaReleases;
}

export async function getMagmaReleaseByTag(
    repo: string,
    tag: string,
): Promise<MagmaRelease> {
    const release = await getReleaseByTag(repo, tag);
    if (release == null) {
        return null;
    }

    const gitCommitUrl = `https://gitlab.com/magmafoundation/${repo}/-/commit/${release.tag_name}`;

    if (release.assets.links[0].url.includes("server")) {
        return {
            name: release.tag_name,
            tag_name: release.tag_name,
            created_at: release.created_at,
            link: release.assets.links[0].url,
            installer_link: release.assets.links[1]?.url,
            git_commit_url: gitCommitUrl,
            archived: await getArchived(repo),
        };
    } else {
        return {
            name: release.tag_name,
            tag_name: release.tag_name,
            created_at: release.created_at,
            link: release.assets.links[0].url,
            installer_link: release.assets.links[0].url,
            git_commit_url: gitCommitUrl,
            archived: await getArchived(repo),
        };
    }
}

type stats = {
    all: number;
    closed: number;
    opened: number;
};

export async function getIssuesStatistics(): Promise<stats> {
    return await remember<stats>("gh:issuestats", async () => {
        console.log("Fetching issues statistics...");
        const stats = await gitlab.IssuesStatistics.all({
            groupId: "magmafoundation",
        });

        return stats.statistics.counts;
    });
}
